@@ -81,6 +81,7 @@ struct SerialThread {
     allow_requesting_clock_sync: bool,
     callback: Box<dyn FnMut(Option<ClockModel>)>,
     triggerbox_data_tx: Option<Sender<TriggerClockInfoRow>>,
+    max_acceptable_measurement_error: Duration,
 }
 
 #[derive(Debug, Clone)]
@@ -119,6 +120,7 @@ impl SerialThread {
         outq: Receiver<Cmd>,
         callback: Box<dyn FnMut(Option<ClockModel>)>,
         triggerbox_data_tx: Option<Sender<TriggerClockInfoRow>>,
+        max_acceptable_measurement_error: Duration,
     ) -> Result<Self> {
         let now = chrono::Utc::now();
         let vquery_time = now + Duration::seconds(1);
@@ -136,6 +138,7 @@ impl SerialThread {
             allow_requesting_clock_sync: false,
             callback,
             triggerbox_data_tx,
+            max_acceptable_measurement_error,
         })
     }
 
@@ -368,8 +371,8 @@ impl SerialThread {
         trace!("this query has send_timestamp: {}", send_timestamp);
 
         let max_error = now.signed_duration_since(send_timestamp);
-        if max_error > Duration::milliseconds(20) {
-            warn!("clock sample took {:?}. Ignoring value.", max_error);
+        if max_error > self.max_acceptable_measurement_error {
+            debug!("clock sample took {:?}. Ignoring value.", max_error);
             return Ok(());
         }
 
@@ -411,27 +414,9 @@ impl SerialThread {
                 ));
 
                 if self.past_data.len() >= 5 {
-                    use na::{OMatrix, OVector, U2};
-
-                    // fit time model
-                    let mut a: Vec<f64> = Vec::with_capacity(self.past_data.len() * 2);
-                    let mut b: Vec<f64> = Vec::with_capacity(self.past_data.len());
-
-                    for row in self.past_data.iter() {
-                        a.push(row.0);
-                        a.push(1.0);
-                        b.push(row.1);
-                    }
-                    let a = OMatrix::<f64, na::Dynamic, U2>::from_row_slice(&a);
-                    let b = OVector::<f64, na::Dynamic>::from_row_slice(&b);
-
-                    let epsilon = 1e-10;
-                    let results = lstsq::lstsq(&a, &b, epsilon)
+                    let (gain, offset, residuals) = fit_time_model(&self.past_data)
                         .map_err(|e| anyhow::anyhow!("lstsq err: {}", e))?;
 
-                    let gain = results.solution[0];
-                    let offset = results.solution[1];
-                    let residuals = results.residuals;
                     let n_measurements = self.past_data.len() as u64;
                     let per_point_residual = residuals / n_measurements as f64;
                     // TODO only accept this if residuals less than some amount?
@@ -510,6 +495,45 @@ impl SerialThread {
     }
 }
 
+fn fit_time_model(past_data: &[(f64, f64)]) -> Result<(f64, f64, f64), &'static str> {
+    use na::{OMatrix, OVector, U2};
+
+    let mut a: Vec<f64> = Vec::with_capacity(past_data.len() * 2);
+    let mut b: Vec<f64> = Vec::with_capacity(past_data.len());
+
+    for row in past_data.iter() {
+        a.push(row.0);
+        a.push(1.0);
+        b.push(row.1);
+    }
+    let a = OMatrix::<f64, na::Dynamic, U2>::from_row_slice(&a);
+    let b = OVector::<f64, na::Dynamic>::from_row_slice(&b);
+
+    let epsilon = 1e-10;
+    let results = lstsq::lstsq(&a, &b, epsilon)?;
+
+    let gain = results.solution[0];
+    let offset = results.solution[1];
+    let residuals = results.residuals;
+
+    Ok((gain, offset, residuals))
+}
+
+#[test]
+fn test_fit_time_model() {
+    let epsilon = 1e-12;
+
+    let data = vec![(0.0, 0.0), (1.0, 1.0), (2.0, 2.0), (3.0, 3.0)];
+    let (gain, offset, _residuals) = fit_time_model(&data).unwrap();
+    assert!((gain - 1.0).abs() < epsilon);
+    assert!((offset - 0.0).abs() < epsilon);
+
+    let data = vec![(0.0, 12.0), (1.0, 22.0), (2.0, 32.0), (3.0, 42.0)];
+    let (gain, offset, _residuals) = fit_time_model(&data).unwrap();
+    assert!((gain - 10.0).abs() < epsilon);
+    assert!((offset - 12.0).abs() < epsilon);
+}
+
 pub fn launch_background_thread(
     callback: Box<dyn FnMut(Option<ClockModel>) + Send>,
     device: String,
@@ -517,14 +541,20 @@ pub fn launch_background_thread(
     triggerbox_data_tx: Option<Sender<TriggerClockInfoRow>>,
     query_dt: std::time::Duration,
     assert_device_name: NameType,
+    max_acceptable_measurement_error: std::time::Duration,
 ) -> Result<(thread_control::Control, std::thread::JoinHandle<()>)> {
     let triggerbox_thread_builder =
         std::thread::Builder::new().name("triggerbox_comms".to_string());
     let (flag, control) = thread_control::make_pair();
     let triggerbox_thread_handle = triggerbox_thread_builder.spawn(move || {
         run_func(|| {
-            let mut triggerbox =
-                SerialThread::new(device, /*raw_tx,*/ cmd, callback, triggerbox_data_tx)?;
+            let mut triggerbox = SerialThread::new(
+                device,
+                /*raw_tx,*/ cmd,
+                callback,
+                triggerbox_data_tx,
+                Duration::from_std(max_acceptable_measurement_error)?,
+            )?;
             triggerbox.run(flag, query_dt, assert_device_name)
         });
     })?;
@@ -562,18 +592,26 @@ fn get_rate(rate_ideal: f64, prescaler: Prescaler) -> (u16, f64) {
     (new_icr1, rate_actual)
 }
 
-pub fn make_trig_fps_cmd(rate_ideal: f64) -> Cmd {
+/// Given an ideal frame rate (in frames per second), compute the triggerbox
+/// command which best approximates this frame rate.
+///
+/// Returns the triggerbox command and the expected actual frame rate (in frames
+/// per second).
+pub fn make_trig_fps_cmd(rate_ideal: f64) -> (Cmd, f64) {
     let (icr1_8, rate_actual_8) = get_rate(rate_ideal, Prescaler::Scale8);
     let (icr1_64, rate_actual_64) = get_rate(rate_ideal, Prescaler::Scale64);
 
     let error_8 = (rate_ideal - rate_actual_8).abs();
     let error_64 = (rate_ideal - rate_actual_64).abs();
 
-    let (icr1, _rate_actual, prescaler) = if error_8 < error_64 {
+    let (icr1, rate_actual, prescaler) = if error_8 < error_64 {
         (icr1_8, rate_actual_8, Prescaler::Scale8)
     } else {
         (icr1_64, rate_actual_64, Prescaler::Scale64)
     };
 
-    Cmd::Icr1AndPrescaler(Icr1AndPrescaler { icr1, prescaler })
+    (
+        Cmd::Icr1AndPrescaler(Icr1AndPrescaler { icr1, prescaler }),
+        rate_actual,
+    )
 }
